@@ -4,13 +4,13 @@
 # either version 2, or (at your option) any later version.
 
 
+import glob
 import os
 import re
 import shutil
 import subprocess
 import sys
 import tempfile
-from urllib.parse import quote_plus
 from urllib.request import HTTPError
 
 from . import checker as osc_checker
@@ -45,23 +45,23 @@ class Fetcher:
         self.gr = OscFileGrabber(progress_obj=self.progress_obj)
 
     def __add_cpio(self, pac):
-        prpap = '%s/%s/%s/%s' % (pac.project, pac.repository, pac.repoarch, pac.repopackage)
+        prpap = f'{pac.project}/{pac.repository}/{pac.repoarch}/{pac.repopackage}'
         self.cpio.setdefault(prpap, {})[pac.repofilename] = pac
 
     def __download_cpio_archive(self, apiurl, project, repo, arch, package, **pkgs):
         if not pkgs:
             return
-        query = ['binary=%s' % quote_plus(i) for i in pkgs]
-        query.append('view=cpio')
-        for module in self.modules:
-            query.append('module=' + module)
+        query = {}
+        query["binary"] = pkgs
+        query["view"] = "cpio"
+        query["module"] = self.modules
         try:
             url = makeurl(apiurl, ['build', project, repo, arch, package], query=query)
             sys.stdout.write("preparing download ...\r")
             sys.stdout.flush()
             with tempfile.NamedTemporaryFile(prefix='osc_build_cpio') as tmparchive:
                 self.gr.urlgrab(url, filename=tmparchive.name,
-                                text='fetching packages for \'%s\'' % project)
+                                text=f'fetching packages for \'{project}\'')
                 archive = cpio.CpioRead(tmparchive.name)
                 archive.read()
                 for hdr in archive:
@@ -133,7 +133,7 @@ class Fetcher:
         mg = OscMirrorGroup(self.gr, pac.urllist)
 
         if self.http_debug:
-            print('\nURLs to try for package \'%s\':' % pac, file=sys.stderr)
+            print(f'\nURLs to try for package \'{pac}\':', file=sys.stderr)
             print('\n'.join(pac.urllist), file=sys.stderr)
             print(file=sys.stderr)
 
@@ -141,7 +141,7 @@ class Fetcher:
             with tempfile.NamedTemporaryFile(prefix='osc_build',
                                              delete=False) as tmpfile:
                 mg_stat = mg.urlgrab(pac.filename, filename=tmpfile.name,
-                                     text='%s(%s) %s' % (prefix, pac.project, pac.filename))
+                                     text=f'{prefix}({pac.project}) {pac.filename}')
                 if mg_stat:
                     self.move_package(tmpfile.name, pac.localdir, pac)
 
@@ -198,7 +198,7 @@ class Fetcher:
         if self.download_api_only:
             return []
         urllist = self.urllist
-        key = '%s/%s' % (pac.project, pac.repository)
+        key = f'{pac.project}/{pac.repository}'
         project_repo_url = buildinfo.urls.get(key)
         if project_repo_url is not None:
             urllist = [project_repo_url]
@@ -289,49 +289,47 @@ class Fetcher:
         self.__fetch_cpio(buildinfo.apiurl)
 
         prjs = list(buildinfo.projects.keys())
-        for i in prjs:
-            dest = "%s/%s" % (self.cachedir, i)
-            if not os.path.exists(dest):
-                os.makedirs(dest, mode=0o755)
-            dest += '/_pubkey'
+        for prj in prjs:
+            dest = os.path.join(self.cachedir, prj)
+            pubkey_path_base = os.path.join(dest, "_pubkey")
+            pubkey_paths = glob.glob(f"{pubkey_path_base}*")
+
+            if self.offline:
+                # we're offline, only index the keys found on disk
+                if pubkey_paths:
+                    for pubkey_path in pubkey_paths:
+                        buildinfo.keys.append(pubkey_path)
+                    buildinfo.prjkeys.append(prj)
+                continue
+
+            from . import obs_api
+
+            os.makedirs(dest, mode=0o755, exist_ok=True)
+            pubkeys = []
 
-            url = makeurl(buildinfo.apiurl, ['source', i, '_pubkey'])
-            try_parent = False
             try:
-                if self.offline and not os.path.exists(dest):
-                    # may need to try parent
-                    try_parent = True
-                elif not self.offline:
-                    OscFileGrabber().urlgrab(url, dest)
-                # not that many keys usually
-                if i not in buildinfo.prjkeys and not try_parent:
-                    buildinfo.keys.append(dest)
-                    buildinfo.prjkeys.append(i)
-            except KeyboardInterrupt:
-                print('Cancelled by user (ctrl-c)')
-                print('Exiting.')
-                if os.path.exists(dest):
-                    os.unlink(dest)
-                sys.exit(0)
+                keyinfo = obs_api.Keyinfo.from_api(buildinfo.apiurl, prj)
+                for pubkey in keyinfo.pubkey_list or []:
+                    pubkeys.append(pubkey.value)
             except HTTPError as e:
-                # Not found is okay, let's go to the next project
-                if e.code != 404:
-                    print("Invalid answer from server", e, file=sys.stderr)
-                    sys.exit(1)
-                try_parent = True
+                result = obs_api.Keyinfo.get_pubkey_deprecated(buildinfo.apiurl, prj, traverse=True)
+                if result:
+                    # overwrite ``prj`` with the project that contains the key we're using
+                    prj, pubkey = result
+                    pubkeys.append(pubkey)
 
-            if try_parent:
-                if self.http_debug:
-                    print("can't fetch key for %s" % (i), file=sys.stderr)
-                    print("url: %s" % url, file=sys.stderr)
+            # remove the existing files, we'll create new files with new contents
+            for pubkey_path in pubkey_paths:
+                os.unlink(pubkey_path)
 
-                if os.path.exists(dest):
-                    os.unlink(dest)
-
-                l = i.rsplit(':', 1)
-                # try key from parent project
-                if len(l) > 1 and l[1] and not l[0] in buildinfo.projects:
-                    prjs.append(l[0])
+            if pubkeys:
+                for num, pubkey in enumerate(pubkeys):
+                    pubkey_path = f"{pubkey_path_base}-{num}"
+                    with open(pubkey_path, "w") as f:
+                        f.write(pubkey)
+                    buildinfo.keys.append(pubkey_path)
+                if prj not in buildinfo.prjkeys:
+                    buildinfo.prjkeys.append(prj)
 
 
 def verify_pacs_old(pac_list):
