@@ -2,13 +2,19 @@ import os
 import sys
 
 
-IS_INTERACTIVE = os.isatty(sys.stdout.fileno())
+try:
+    IS_INTERACTIVE = os.isatty(sys.stdout.fileno())
+except OSError:
+    IS_INTERACTIVE = False
 
 
 ESCAPE_CODES = {
     "reset": "\033[0m",
     "bold": "\033[1m",
+    "dim": "\033[2m",
+    "italic": "\033[3m",
     "underline": "\033[4m",
+    "blink": "\033[5m",
     "black": "\033[30m",
     "red": "\033[31m",
     "green": "\033[32m",
@@ -17,6 +23,14 @@ ESCAPE_CODES = {
     "magenta": "\033[35m",
     "cyan": "\033[36m",
     "white": "\033[37m",
+    "bg_black": "\033[40m",
+    "bg_red": "\033[41m",
+    "bg_green": "\033[42m",
+    "bg_yellow": "\033[43m",
+    "bg_blue": "\033[44m",
+    "bg_magenta": "\033[45m",
+    "bg_cyan": "\033[46m",
+    "bg_white": "\033[47m",
 }
 
 
@@ -30,6 +44,9 @@ def colorize(text, color):
     if not color:
         return text
 
+    if not text:
+        return text
+
     result = ""
     for i in color.split(","):
         result += ESCAPE_CODES[i]
