@@ -1,19 +1,27 @@
-
-
-
 import os
 import re
 import struct
+import sys
 from . import packagequery
 
+from .helper import decode_it
+
+
+def cmp(a, b):
+    return (a > b) - (a < b)
+
+
 class RpmError(packagequery.PackageError):
     pass
 
+
 class RpmHeaderError(RpmError):
     pass
 
+
 class RpmHeader:
     """corresponds more or less to the indexEntry_s struct"""
+
     def __init__(self, offset, length):
         self.offset = offset
         # length of the data section (without length of indexEntries)
@@ -30,17 +38,18 @@ class RpmHeader:
         return None
 
     def __iter__(self):
-        for i in self.entries:
-            yield i
+        yield from self.entries
 
     def __len__(self):
         return len(self.entries)
 
+
 class RpmHeaderEntry:
     """corresponds to the entryInfo_s struct (except the data attribute)"""
 
     # each element represents an int
     ENTRY_SIZE = 16
+
     def __init__(self, tag, type, offset, count):
         self.tag = tag
         self.type = type
@@ -48,6 +57,7 @@ class RpmHeaderEntry:
         self.count = count
         self.data = None
 
+
 class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
     LEAD_SIZE = 96
     LEAD_MAGIC = 0xedabeedb
@@ -58,11 +68,19 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
     GREATER = 1 << 2
     EQUAL = 1 << 3
 
-    default_tags = (1000, 1001, 1002, 1003, 1004, 1022, 1005, 1020,
-        1047, 1112, 1113, # provides
-        1049, 1048, 1050, # requires
-        1054, 1053, 1055, # conflicts
-        1090, 1114, 1115  # obsoletes
+    SENSE_STRONG = 1 << 27
+
+    default_tags = (
+        1000, 1001, 1002, 1003, 1004, 1022, 1005, 1020,
+        1047, 1112, 1113,  # provides
+        1049, 1048, 1050,  # requires
+        1054, 1053, 1055,  # conflicts
+        1090, 1114, 1115,  # obsoletes
+        1156, 1158, 1157,  # oldsuggests
+        5046, 5047, 5048,  # recommends
+        5049, 5051, 5050,  # suggests
+        5052, 5053, 5054,  # supplements
+        5055, 5056, 5057  # enhances
     )
 
     def __init__(self, fh):
@@ -98,16 +116,17 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         data = self.__file.read(self.header.length)
         for i in self.header:
             if i.tag in self.default_tags + extra_tags or all_tags:
-                try: # this may fail for -debug* packages
+                try:  # this may fail for -debug* packages
                     self.__read_data(i, data)
-                except: pass
+                except:
+                    pass
         return self
 
     def __read_lead(self):
         data = self.__file.read(self.LEAD_SIZE)
         leadmgc, = struct.unpack('!I', data[:4])
         if leadmgc != self.LEAD_MAGIC:
-            raise RpmError(self.__path, 'invalid lead magic \'%s\'' % leadmgc)
+            raise RpmError(self.__path, 'not a rpm (invalid lead magic \'%s\')' % leadmgc)
         sigtype, = struct.unpack('!h', data[78:80])
         if sigtype != self.HEADERSIG_TYPE:
             raise RpmError(self.__path, 'invalid header signature \'%s\'' % sigtype)
@@ -156,7 +175,7 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         else:
             raise RpmHeaderError(self.__path, 'unsupported tag type \'%d\' (tag: \'%s\'' % (entry.type, entry.tag))
 
-    def __reqprov(self, tag, flags, version):
+    def __reqprov(self, tag, flags, version, strong=None):
         pnames = self.header.gettag(tag)
         if not pnames:
             return []
@@ -167,16 +186,24 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
             raise RpmError(self.__path, 'cannot get provides/requires, tags are missing')
         res = []
         for name, flags, ver in zip(pnames, pflags, pvers):
+            if strong is not None:
+                # compat code for the obsolete RPMTAG_OLDSUGGESTSNAME tag
+                # strong == 1 => return only "recommends"
+                # strong == 0 => return only "suggests"
+                if strong == 1:
+                    strong = self.SENSE_STRONG
+                if (flags & self.SENSE_STRONG) != strong:
+                    continue
             # RPMSENSE_SENSEMASK = 15 (see rpmlib.h) but ignore RPMSENSE_SERIAL (= 1 << 0) therefore use 14
             if flags & 14:
-                name += ' '
+                name += b' '
                 if flags & self.GREATER:
-                    name += '>'
+                    name += b'>'
                 elif flags & self.LESS:
-                    name += '<'
+                    name += b'<'
                 if flags & self.EQUAL:
-                    name += '='
-                name += ' %s' % ver
+                    name += b'='
+                name += b' %s' % ver
             res.append(name)
         return res
 
@@ -236,6 +263,24 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
     def obsoletes(self):
         return self.__reqprov(1090, 1114, 1115)
 
+    def recommends(self):
+        recommends = self.__reqprov(5046, 5048, 5047)
+        if not recommends:
+            recommends = self.__reqprov(1156, 1158, 1157, 1)
+        return recommends
+
+    def suggests(self):
+        suggests = self.__reqprov(5049, 5051, 5050)
+        if not suggests:
+            suggests = self.__reqprov(1156, 1158, 1157, 0)
+        return suggests
+
+    def supplements(self):
+        return self.__reqprov(5052, 5054, 5053)
+
+    def enhances(self):
+        return self.__reqprov(5055, 5057, 5506)
+
     def is_src(self):
         # SOURCERPM = 1044
         return self.gettag(1044) is None
@@ -250,9 +295,9 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
 
     def canonname(self):
         if self.is_nosrc():
-            arch = 'nosrc'
+            arch = b'nosrc'
         elif self.is_src():
-            arch = 'src'
+            arch = b'src'
         else:
             arch = self.arch()
         return RpmQuery.filename(self.name(), None, self.version(), self.release(), arch)
@@ -274,7 +319,7 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         entry = rpmq.gettag(1004)
         if entry is None:
             return None
-        return ''.join([ "%02x" % x for x in struct.unpack('16B', entry.data) ])
+        return ''.join(["%02x" % x for x in struct.unpack('16B', entry.data)])
 
     @staticmethod
     def rpmvercmp(ver1, ver2):
@@ -285,12 +330,24 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         if ver1 == ver2:
             return 0
         res = 0
+        ver1 = decode_it(ver1)
+        ver2 = decode_it(ver2)
         while res == 0:
-            # remove all leading non alphanumeric chars
-            ver1 = re.sub('^[^a-zA-Z0-9]*', '', ver1)
-            ver2 = re.sub('^[^a-zA-Z0-9]*', '', ver2)
+            # remove all leading non alphanumeric or tilde chars
+            ver1 = re.sub('^[^a-zA-Z0-9~]*', '', ver1)
+            ver2 = re.sub('^[^a-zA-Z0-9~]*', '', ver2)
+            if ver1.startswith('~') or ver2.startswith('~'):
+                if not ver1.startswith('~'):
+                    return 1
+                elif not ver2.startswith('~'):
+                    return -1
+                ver1 = ver1[1:]
+                ver2 = ver2[1:]
+                continue
+
             if not (len(ver1) and len(ver2)):
                 break
+
             # check if we have a digits segment
             mo1 = re.match(r'(\d+)', ver1)
             mo2 = re.match(r'(\d+)', ver2)
@@ -325,21 +382,21 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
 
     @staticmethod
     def filename(name, epoch, version, release, arch):
-        return '%s-%s-%s.%s.rpm' % (name, version, release, arch)
+        return b'%s-%s-%s.%s.rpm' % (name, version, release, arch)
 
-def unpack_string(data):
+
+def unpack_string(data, encoding=None):
     """unpack a '\\0' terminated string from data"""
-    val = ''
-    for c in data:
-        c, = struct.unpack('!c', c)
-        if c == '\0':
-            break
-        else:
-            val += c
-    return val
+    idx = data.find(b'\0')
+    if idx == -1:
+        raise ValueError('illegal string: not \\0 terminated')
+    data = data[:idx]
+    if encoding is not None:
+        data = data.decode(encoding)
+    return data
+
 
 if __name__ == '__main__':
-    import sys
     try:
         rpmq = RpmQuery.query(sys.argv[1])
     except RpmError as e:
