@@ -43,7 +43,7 @@ class TestCommit(OscTestCase):
         self.assertEqual(sys.stdout.getvalue(), exp)
         self._check_digests('testSimple_cfilesremote')
         self.assertTrue(os.path.exists('nochange'))
-        self.assertFilesEqual('nochange', os.path.join('.osc', 'nochange'))
+        self.assertFilesEqual('nochange', os.path.join('.osc', 'sources', 'nochange'))
         self._check_status(p, 'nochange', ' ')
         self._check_status(p, 'foo', ' ')
         self._check_status(p, 'merge', ' ')
@@ -67,7 +67,7 @@ class TestCommit(OscTestCase):
         self.assertEqual(sys.stdout.getvalue(), exp)
         self._check_digests('testAddfile_cfilesremote')
         self.assertTrue(os.path.exists('add'))
-        self.assertFilesEqual('add', os.path.join('.osc', 'add'))
+        self.assertFilesEqual('add', os.path.join('.osc', 'sources', 'add'))
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
         self._check_status(p, 'add', ' ')
         self._check_status(p, 'foo', ' ')
@@ -89,7 +89,7 @@ class TestCommit(OscTestCase):
         self.assertEqual(sys.stdout.getvalue(), exp)
         self._check_digests('testDeletefile_cfilesremote')
         self.assertFalse(os.path.exists('nochange'))
-        self.assertFalse(os.path.exists(os.path.join('.osc', 'nochange')))
+        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'nochange')))
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
         self._check_status(p, 'foo', ' ')
         self._check_status(p, 'merge', ' ')
@@ -144,8 +144,8 @@ class TestCommit(OscTestCase):
         self._check_digests('testMultiple_cfilesremote')
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
-        self.assertFalse(os.path.exists(os.path.join('.osc', 'foo')))
-        self.assertFalse(os.path.exists(os.path.join('.osc', 'merge')))
+        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
+        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'merge')))
         self.assertRaises(osc.oscerr.OscIOError, p.status, 'foo')
         self.assertRaises(osc.oscerr.OscIOError, p.status, 'merge')
         self._check_status(p, 'add', ' ')
@@ -221,7 +221,7 @@ class TestCommit(OscTestCase):
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_deleted')))
         self.assertFalse(os.path.exists('foo'))
-        self.assertFalse(os.path.exists(os.path.join('.osc', 'foo')))
+        self.assertFalse(os.path.exists(os.path.join('.osc', 'sources', 'foo')))
         self._check_status(p, 'add', ' ')
         self._check_status(p, 'nochange', ' ')
         self._check_status(p, 'merge', '!')
@@ -244,7 +244,7 @@ class TestCommit(OscTestCase):
         self.assertEqual(sys.stdout.getvalue(), exp)
         self._check_digests('testAddfile_cfilesremote')
         self.assertTrue(os.path.exists('add'))
-        self.assertFilesEqual('add', os.path.join('.osc', 'add'))
+        self.assertFilesEqual('add', os.path.join('.osc', 'sources', 'add'))
         self.assertFalse(os.path.exists(os.path.join('.osc', '_to_be_added')))
         self._check_status(p, 'add', ' ')
         self._check_status(p, 'foo', ' ')
@@ -344,7 +344,7 @@ class TestCommit(OscTestCase):
         self.assertEqual(sys.stdout.getvalue(), exp)
         self._check_digests('testSimple_cfilesremote')
         self.assertTrue(os.path.exists('nochange'))
-        self.assertFilesEqual('nochange', os.path.join('.osc', 'nochange'))
+        self.assertFilesEqual('nochange', os.path.join('.osc', 'sources', 'nochange'))
         self._check_status(p, 'nochange', ' ')
         self._check_status(p, 'foo', ' ')
         self._check_status(p, 'merge', ' ')
