@@ -1,4 +1,3 @@
-import importlib
 import os
 import shutil
 import tempfile
@@ -6,6 +5,8 @@ import unittest
 
 import osc.conf
 
+from .common import patch
+
 
 OSCRC = """
 [general]
@@ -42,6 +43,7 @@ debug = 0
 http_debug = 0
 http_full_debug = 0
 http_retries = 3
+quiet = 0
 verbose = 0
 no_preinstallimage = 0
 traceback = 0
@@ -86,7 +88,7 @@ credentials_mgr_class=osc.credentials.PlaintextConfigFileCredentialsManager
 user = Admin
 pass = opensuse
 passx = unused
-aliases = osc
+aliases = obs
 http_headers = 
     Authorization: Basic QWRtaW46b3BlbnN1c2U=
     X-Foo: Bar
@@ -105,7 +107,6 @@ plugin-option = plugin-host-option
 
 class TestExampleConfig(unittest.TestCase):
     def setUp(self):
-        importlib.reload(osc.conf)
         self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")
         self.oscrc = os.path.join(self.tmpdir, "oscrc")
         with open(self.oscrc, "w", encoding="utf-8") as f:
@@ -116,6 +117,9 @@ class TestExampleConfig(unittest.TestCase):
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+    def test_invalid_attribute(self):
+        self.assertRaises(AttributeError, setattr, self.config, "new_attribute", "123")
+
     def test_apiurl(self):
         self.assertEqual(self.config["apiurl"], "https://api.opensuse.org")
 
@@ -215,6 +219,9 @@ class TestExampleConfig(unittest.TestCase):
     def test_http_retries(self):
         self.assertEqual(self.config["http_retries"], 3)
 
+    def test_quiet(self):
+        self.assertEqual(self.config["quiet"], False)
+
     def test_verbose(self):
         self.assertEqual(self.config["verbose"], False)
 
@@ -407,33 +414,53 @@ class TestExampleConfig(unittest.TestCase):
 
     def test_extra_fields(self):
         self.assertEqual(self.config["plugin-option"], "plugin-general-option")
-        self.assertEqual(self.config.extra_fields, {"plugin-option": "plugin-general-option"})
-
-        # write to an existing attribute instead of extra_fields
-        self.config.attrib = 123
-        self.assertEqual(self.config["attrib"], 123)
-        self.config["attrib"] = 456
-        self.assertEqual(self.config["attrib"], 456)
-        self.assertEqual(self.config.extra_fields, {"plugin-option": "plugin-general-option"})
+        self.assertEqual(self.config._extra_fields, {"plugin-option": "plugin-general-option"})
 
         self.config["new-option"] = "value"
         self.assertEqual(self.config["new-option"], "value")
-        self.assertEqual(self.config.extra_fields, {"plugin-option": "plugin-general-option", "new-option": "value"})
+        self.assertEqual(self.config._extra_fields, {"plugin-option": "plugin-general-option", "new-option": "value"})
 
         host_options = self.config["api_host_options"][self.config["apiurl"]]
         self.assertEqual(host_options["plugin-option"], "plugin-host-option")
-        self.assertEqual(host_options.extra_fields, {"plugin-option": "plugin-host-option"})
+        self.assertEqual(host_options._extra_fields, {"plugin-option": "plugin-host-option"})
 
         host_options["new-option"] = "value"
         self.assertEqual(host_options["new-option"], "value")
-        self.assertEqual(host_options.extra_fields, {"plugin-option": "plugin-host-option", "new-option": "value"})
+        self.assertEqual(host_options._extra_fields, {"plugin-option": "plugin-host-option", "new-option": "value"})
 
     def test_apiurl_aliases(self):
-        expected = {"https://api.opensuse.org": "https://api.opensuse.org", "osc": "https://api.opensuse.org"}
+        expected = {"https://api.opensuse.org": "https://api.opensuse.org", "obs": "https://api.opensuse.org"}
         self.assertEqual(self.config.apiurl_aliases, expected)
         self.assertEqual(self.config["apiurl_aliases"], expected)
 
 
+class TestOverrides(unittest.TestCase):
+    def test_verbose(self):
+        self.options = osc.conf.Options()
+        self.assertEqual(self.options.quiet, False)
+        self.assertEqual(self.options.verbose, False)
+
+        self.options.quiet = True
+        self.options.verbose = True
+        self.assertEqual(self.options.quiet, True)
+        # ``verbose`` is forced to ``False`` by the ``quiet`` option
+        self.assertEqual(self.options.verbose, False)
+
+        self.options.quiet = False
+        self.assertEqual(self.options.quiet, False)
+        self.assertEqual(self.options.verbose, True)
+
+    def test_http_debug(self):
+        self.options = osc.conf.Options()
+        self.assertEqual(self.options.http_debug, False)
+        self.assertEqual(self.options.http_full_debug, False)
+
+        self.options.http_full_debug = True
+        # ``http_debug`` forced to ``True`` by the ``http_full_debug`` option
+        self.assertEqual(self.options.http_debug, True)
+        self.assertEqual(self.options.http_full_debug, True)
+
+
 class TestFromParent(unittest.TestCase):
     def setUp(self):
         self.options = osc.conf.Options()
@@ -469,5 +496,183 @@ class TestFromParent(unittest.TestCase):
         self.assertEqual(self.host_options.email, "another-user@example.com")
 
 
+class TestConf(unittest.TestCase):
+    def setUp(self):
+        self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")
+
+    def tearDown(self):
+        shutil.rmtree(self.tmpdir)
+
+    def test_write_initial_config(self):
+        conffile = os.path.join(self.tmpdir, "oscrc")
+        entries = {
+            "user": "Admin",
+            "pass": "opensuse",
+            "apiurl": "https://example.com",
+        }
+        osc.conf.write_initial_config(conffile, entries)
+
+    def test_api_host_options(self):
+        # test that instances do not share any references leaked from the defaults
+        conf1 = osc.conf.Options()
+        conf2 = osc.conf.Options()
+
+        self.assertEqual(conf1, conf2)  # models are compared by their contents now
+        self.assertNotEqual(id(conf1), id(conf2))
+        self.assertNotEqual(id(conf1.api_host_options), id(conf2.api_host_options))
+
+
+class TestCredentialsFromEnv(unittest.TestCase):
+    def setUp(self):
+        osc.conf.config = None
+        self.oscrc = ""
+
+    @patch.dict(os.environ, {"OSC_APIURL": "https://example.com"}, clear=True)
+    def test_new_apiurl(self):
+        # missing user
+        self.assertRaises(
+            osc.oscerr.ConfigMissingCredentialsError,
+            osc.conf.get_config,
+            override_conffile=self.oscrc,
+        )
+
+    @patch.dict(
+        os.environ,
+        {"OSC_APIURL": "https://example.com", "OSC_USERNAME": "user"},
+        clear=True,
+    )
+    def test_new_apiurl_username(self):
+        # missing password
+        self.assertRaises(
+            osc.oscerr.ConfigMissingCredentialsError,
+            osc.conf.get_config,
+            override_conffile=self.oscrc,
+        )
+
+    @patch.dict(
+        os.environ,
+        {
+            "OSC_APIURL": "https://example.com",
+            "OSC_USERNAME": "user",
+            "OSC_PASSWORD": "secret",
+        },
+        clear=True,
+    )
+    def test_new_apiurl_username_password(self):
+        # missing password
+        osc.conf.get_config(override_conffile=self.oscrc)
+        conf = osc.conf.config
+        host_options = conf["api_host_options"][conf["apiurl"]]
+        self.assertEqual(conf.apiurl, "https://example.com")
+        self.assertEqual(host_options.apiurl, "https://example.com")
+        self.assertEqual(host_options.username, "user")
+        self.assertEqual(host_options.password, "secret")
+        self.assertEqual(host_options.credentials_mgr_class, None)
+
+    @patch.dict(
+        os.environ,
+        {
+            "OSC_APIURL": "https://example.com",
+            "OSC_USERNAME": "user",
+            "OSC_PASSWORD": "secret",
+        },
+        clear=True,
+    )
+    def test_new_apiurl_username_password(self):
+        # missing password
+        osc.conf.get_config(override_conffile=self.oscrc)
+        conf = osc.conf.config
+        host_options = conf["api_host_options"][conf["apiurl"]]
+        self.assertEqual(conf.apiurl, "https://example.com")
+        self.assertEqual(host_options.apiurl, "https://example.com")
+        self.assertEqual(host_options.username, "user")
+        self.assertEqual(host_options.password, "secret")
+        self.assertEqual(host_options.credentials_mgr_class, None)
+
+    @patch.dict(
+        os.environ,
+        {
+            "OSC_APIURL": "https://example.com",
+            "OSC_USERNAME": "user",
+            "OSC_PASSWORD": "secret",
+            "OSC_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
+        },
+        clear=True,
+    )
+    def test_new_apiurl_username_password_credmgr(self):
+        # missing password
+        osc.conf.get_config(override_conffile=self.oscrc)
+        conf = osc.conf.config
+        host_options = conf["api_host_options"][conf.apiurl]
+        self.assertEqual(conf.apiurl, "https://example.com")
+        self.assertEqual(host_options.apiurl, "https://example.com")
+        self.assertEqual(host_options.username, "user")
+        self.assertEqual(host_options.password, "secret")
+        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.PlaintextConfigFileCredentialsManager")
+
+
+class TestHostOptionsFromEnv(unittest.TestCase):
+    def setUp(self):
+        self.tmpdir = tempfile.mkdtemp(prefix="osc_test_")
+        self.oscrc = os.path.join(self.tmpdir, "oscrc")
+        with open(self.oscrc, "w", encoding="utf-8") as f:
+            f.write(OSCRC)
+        osc.conf.get_config(override_conffile=self.oscrc)
+        self.config = osc.conf.config
+
+    def tearDown(self):
+        shutil.rmtree(self.tmpdir)
+
+    @patch.dict(
+        os.environ,
+        {
+            "OSC_HOST_OBS_USERNAME": "user",
+            "OSC_HOST_OBS_PASSWORD": "secret",
+            "OSC_HOST_OBS_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
+            "OSC_HOST_OBS_REALNAME": "User",
+            "OSC_HOST_OBS_EMAIL": "user@example.com",
+        },
+        clear=True,
+    )
+    def test_host_options(self):
+        osc.conf.get_config(override_conffile=self.oscrc)
+        conf = osc.conf.config
+        host_options = conf["api_host_options"][conf["apiurl"]]
+        self.assertEqual(conf.apiurl, "https://api.opensuse.org")
+        self.assertEqual(host_options.apiurl, "https://api.opensuse.org")
+        self.assertEqual(host_options.username, "user")
+        self.assertEqual(host_options.password, "secret")
+        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.PlaintextConfigFileCredentialsManager")
+        self.assertEqual(host_options.realname, "User")
+        self.assertEqual(host_options.email, "user@example.com")
+
+    @patch.dict(
+        os.environ,
+        {
+            "OSC_HOST_OBS_USERNAME": "user",
+            "OSC_HOST_OBS_PASSWORD": "secret",
+            "OSC_HOST_OBS_CREDENTIALS_MGR_CLASS": "osc.credentials.PlaintextConfigFileCredentialsManager",
+            "OSC_HOST_OBS_REALNAME": "User",
+            "OSC_HOST_OBS_EMAIL": "user@example.com",
+            "OSC_USERNAME": "USER",
+            "OSC_PASSWORD": "SECRET",
+            "OSC_CREDENTIALS_MGR_CLASS": "osc.credentials.TransientCredentialsManager",
+        },
+        clear=True,
+    )
+    def test_host_options_overrides(self):
+        # thest if OSC_{USERNAME,PASSWORD,CREDENTIALS_MGR_CLASS} prevail over OSC_HOST_* options
+        osc.conf.get_config(override_conffile=self.oscrc)
+        conf = osc.conf.config
+        host_options = conf["api_host_options"][conf["apiurl"]]
+        self.assertEqual(conf.apiurl, "https://api.opensuse.org")
+        self.assertEqual(host_options.apiurl, "https://api.opensuse.org")
+        self.assertEqual(host_options.username, "USER")
+        self.assertEqual(host_options.password, "SECRET")
+        self.assertEqual(host_options.credentials_mgr_class, "osc.credentials.TransientCredentialsManager")
+        self.assertEqual(host_options.realname, "User")
+        self.assertEqual(host_options.email, "user@example.com")
+
+
 if __name__ == "__main__":
     unittest.main()
